CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     THIS PROGRAM PERFORMS A FORMATTING OF THE GEOMETRICAL DATA OF A FLAC3D-GENERATED             C
C     MESH, IN ORDER TO MAKE TOUGH2/iTOUGH2 UNDERSTAND IT.                                         C
C     STEPS TO USE THIS PROGRAM:                                                                   C
C     1. GENERATION OF THE MESH USING THE FLAC3D BUILD-IN TOOLS (i.e., COMMAND "GEN ZONE ...").    C
C        NOTE THAT A GROUP NUMBER MUST BE ASSIGNED TO EACH ZONE, WITHIN THE COMMAND "GEN ZONE"     C
C        (SEE THE COMMAND REFERENCE MANUAL FOR DETAILS). THE GROUP NUMBER MUST BE THE SAME THAN    C
C        THE CORRESPONDING MATERIAL INDEX IN BLOCK ROCKS IN TOUGH2, MATX(ZONE). THE MESH           C
C        GENERATION SHOULD BE DONE AT THE CORRESPONDING POSITION IN FILE "MESH_GENERATION.F3DAT",  C
C        TO BE OPENED FROM "MESH_FLATOU.F3PRJ";                                                    C
C     3. EXECUTION (IN FLAC3D) OF FILE MESH_GENERATION.F3DAT. FILE "mesh_export.txt" IS CREATED;   C
C     4. EXECUTION OF THIS PROGRAM (MESH_FLATOU.EXE), (FILE "mesh_export.txt" MUST BE IN THE SAME  C
C        FOLDER). FILE "MESH" IS CREATED, WHICH IS TO BE GIVEN TO TOUGH2/iT2 AS INPUT; THEREFORE,  C
C        BLOCKS "ELEME" AND "CONNE" ARE NOT NECESSARY IN THE INPUT FILE.                           C
C        NOTE THAT THE VOLUME OF SOME ELEMENTS MIGHT NEED TO BE MODIFIED BY HAND, IF FOR INSTANCE  C
C        DIRICHLET BOUNDARY CONDITIONS ARE USED;                                                   C
C     5. ADDITIONALLY, THIS PROGRAM CAN ALSO CREATE A FILE INCON, ACCORDING TO THE EOS3 (AT LEAST) C
C        FORMATTING. HOWEVER, THE PRESSURE, TEMPERATURE AND GAS SATURATION FUNCTIONS HAVE TO BE    C
C        MODIFIED TO FIT EACH CASE. ALTERNATIVELY, THE CREATION OF FILE INCON CAN BE CANCELLED BY  C
C        SIMPLY COMMENTING THE CORRESPONDING LINES IN THIS FILE.                                   C
C**************************************************************************************************C
C     NOTES ON THE NOMENCLATURE:                                                                   C
C     DEPENDING ON THE SHAPE OF THE MODEL AND THE WAY THE MESH IS CREATED, IT IS UNLIKELY THAT     C
C     THE ZONES CREATED IN FLAC3D HAVE ALL A POSITION-LOGIC ID.                                    C
C     IN ORDER TO MAKE THE FLAC3D AND THE TOUGH2 MESHES SOMEWHAT RELATED IN TERMS OF ELEMENTS      C
C     NAMING, AND BEARING THE TOUGH2 RESTRICTIONS INTO ACCOUNT (INPUT FORMAT MUST BE (A3,I2) IF    C
C     AT LEAST THE ELEME&CONNE OR THE INCON DATA ARE SUPPLIED IN THE INPUT FILE), THE FOLLOWING    C
C     RULES HAVE BEEN USED IN THIS PROGRAM. THESE RULES ARE INTENDED TO HELP THE USER FIND A       C
C     SPECIFIC ZONE (OR GRID BLOCK) IN THE TWO CODES, FOR INITIAL CONDITIONS DEFINITION OR ANY     C
C     OTHER PURPOSE.                                                                               C
C     1. THE LAST TWO DIGITS ARE ALWAYS INTEGERS, EQUAL TO THE LAST TWO DIGITS OF THE ZONE ID IN   C
C        FLAC3D. IF THE TENTHS DIGIT IS A ZERO, THAT DIGIT WILL NOT BE WRITTEN;                    C
C     2. THE HUNDREDTHS DIGIT IS A LETTER, FROM "A" TO "I", CORRESPONDING TO 1->9. IF THE ZONE ID  C
C        IS BIGGER THAN 999 AND THE HUNDREDTHS DIGIT IS ZERO (E.G. 5024), AN EMPTY CHARACTER WILL  C
C        BE WRITTEN. ACCORDING TO RULES 1 AND 2, ZONE "5024" IN FLAC3D WOULD BE " E 24" IN TOUGH2; C
C     3. IDEM FOR THE THOUSANDTHS DIGIT;                                                           C
C     4. FOR ZONE IDS WITH 6 DIGITS, THE FIRST CHARACTER OF THE GRID BLOCK NAME IN TOUGH2          C
C        IS DETERMINED AS FOLLOWS: FROM 1 TO 26, THE CORRESPONDING LETTER A->Z IS USED, AND FROM   C
C        27 TO 35, NUMBERS 1->9 ARE USED. THEREFORE, THE MAXIMUM NUMBER OF ZONES IS 359,999.       C
C     AS A GENERAL RULE, WHEN A DIGIT IS ZERO IN FLAC3D, IT WILL NOT BE WRITTEN IN THE GRID BLOCK  C
C     ID IN TOUGH2 (EXCEPT FOR THE UNITS POSITION).                                                C
C     EXAMPLES:                                                                                    C
C     56 ->"   56"                                                                                 C
C     845 -> "  H45"                                                                               C
C     1259 -> " AB59"                                                                              C
C     10907 -> "A I07"                                                                             C
C     354389 -> "9DC89"                                                                            C
C**************************************************************************************************C
C     LAURA BLANCO MARTIN, MARCH 2013.                                                             C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      PROGRAM MESH_FLATOU
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      CHARACTER*80 A
      CHARACTER*3 NEW_N,NEW_N1,NEW_N2
      OPEN(UNIT=1,FILE='mesh_export.txt',FORM='FORMATTED',STATUS='OLD')
      OPEN(UNIT=2,FILE='MESH',FORM='FORMATTED',STATUS='UNKNOWN')
C      OPEN(UNIT=3,FILE='INCON',FORM='FORMATTED',STATUS='UNKNOWN')
      READ(1,*) NEL,NCON
      READ(1,'(A)') A
      WRITE(2,'(A5)') 'ELEME'
C      WRITE(3,'(A5)') 'INCON'
      DO 1 I=1,NEL
          READ(1,*) N,M,VOLX,AHTX,X,Y,Z
          MI=MOD(N,100)
          CALL ID_CONVERSION(N,NEW_N)
          WRITE(2,100) NEW_N,MI,M,VOLX,AHTX,0.,X,Y,Z
C          WRITE(3,104) NEW_N,MI
C          P=100000.-9.81*1000.*(Z-600.)
C          IF(M.EQ.2.OR.M.EQ.3) P=100028.
C          SG=0.1E-4
C          IF(M.EQ.2) SG=10.95
C          T=28.-0.03*Z
C          WRITE(3,105) P,SG,T
    1 CONTINUE
      READ(1,'(A)') A
      WRITE(2,101)
      DO 2 J=1,NCON
          READ(1,*) N1,N2,ISOX,D1,D2,AREAX,BETAX
          M1=MOD(N1,100)
          CALL ID_CONVERSION(N1,NEW_N1)
          M2=MOD(N2,100)
          CALL ID_CONVERSION(N2,NEW_N2)
          WRITE(2,102) NEW_N1,M1,NEW_N2,M2,ISOX,D1,D2,AREAX,BETAX,0.
    2 CONTINUE
      WRITE(2,'(A)') ' '
  100 FORMAT(A3,I2,10X,I5,6E10.4)
  101 FORMAT(' '/'CONNE')
  102 FORMAT(2(A3,I2),15X,I5,5E10.4)
C  104 FORMAT(A3,I2)
C  105 FORMAT(3E20.14) 
      ENDFILE 2
C      ENDFILE 3
      END
      SUBROUTINE ID_CONVERSION(N,NEW_N)
      CHARACTER*1 NA,NI,NII,NIII
      CHARACTER*3 NEW_N
      DIMENSION NA(35)
      DATA NA/'A','B','C','D','E','F','G','H','I','J','K','L','M',
     >'N','O','P','Q','R','S','T','U','V','W','X','Y','Z','1','2','3',
     >'4','5','6','7','8','9'/
      NI=' '
      NII=' '
      NIII=' '
      IF(N.GE.100.AND.N.LT.1000) NI=NA(N/100)
      IF(N.GE.1000.AND.N.LT.10000) THEN
          NII=NA(N/1000)
          IF(MOD(N,1000).GE.100) NI=NA(MOD(N,1000)/100)
      ENDIF
      IF(N.GT.359999) THEN
          WRITE(2,103) N
          STOP
      ENDIF
      IF(N.GE.10000) THEN
          NIII=NA(N/10000)
          IF(MOD(N,10000).GE.1000) NII=NA(MOD(N,10000)/1000)
          IF(MOD(N,1000).GE.100) NI=NA(MOD(N,1000)/100)
      ENDIF
      NEW_N=NIII//NII//NI
  103 FORMAT('NUMBER OF ELEMENTS READ SO FAR, ',I6,', EXCEEDS ALLOWABLE 
     >MAXIMUM OF 359999 ELEMENTS.'/'MODIFICATION OF THIS PROGRAM IS 
     >NECESSARY TO CONTINUE.')      
      END